//
//  UserService.swift
//  TestMe
//
//  Created by Stefan Popp on 20.04.16.
//  Copyright © 2016 Stefan Mayer-Popp. All rights reserved.
//

import Foundation

import Alamofire
import ObjectMapper
import AlamofireObjectMapper

class User : Mappable {
    var username: String?
    
    required init?(_ map: Map) {
        
    }
    
    func mapping(map: Map) {
        username <- map["username"]
    }
}

enum UserServiceError {
    case NotFound
    case WrongAPI
    case UnknownError
}

enum UserServiceResult<T> {
    case Success(T)
    case Error(UserServiceError)
}

enum UserService {
    static func getUserList(completion completion: (UserServiceResult<[User]>) -> Void) {
        Alamofire.request(.GET, NSURL(string: "https://www.google.de/listUsers")!)
            .responseArray { (response: Alamofire.Response<[User], NSError>) in
                
                guard let httpResponse = response.response where httpResponse.statusCode == 200 else {
                    completion(.Error(.UnknownError))
                    return
                }
                
                guard let userList = response.result.value else {
                    completion(.Error(.NotFound))
                    return
                }
                
                completion(.Success(userList))
        }
    }
}