//
//  TestMeTests.swift
//  TestMeTests
//
//  Created by Stefan Popp on 20.04.16.
//  Copyright © 2016 Stefan Mayer-Popp. All rights reserved.
//

import XCTest

import Mockingjay

@testable import TestMe

class TestMeTests: XCTestCase {
    
    override func setUp() {
        super.setUp()
        // Put setup code here. This method is called before the invocation of each test method in the class.
    }
    
    override func tearDown() {
        // Put teardown code here. This method is called after the invocation of each test method in the class.
        super.tearDown()
    }
    
    func testUnknownHTTPError() {
        // This is an example of a functional test case.
        // Use XCTAssert and related functions to verify your tests produce the correct results.
        stub({ (request) -> (Bool) in
            if request.URL!.path!.hasPrefix("/listUsers") {
                return true
            }
            return false
            }, builder: http(401))
        
        let expectation = expectationWithDescription("Async")
        UserService.getUserList { (serviceResult) in
            guard case .Error(let error) = serviceResult where error == .UnknownError else {
                XCTAssert(false, "Should return unknown error")
                return
            }
            expectation.fulfill()
        }
        waitForExpectationsWithTimeout(5.0, handler: nil)
    }
    
    func testEmptyResult() {
        // This is an example of a functional test case.
        // Use XCTAssert and related functions to verify your tests produce the correct results.
        stub({ (request) -> (Bool) in
            if request.URL!.path!.hasPrefix("/listUsers") {
                return true
            }
            return false
            }, builder: http(200))
        
        let expectation = expectationWithDescription("Async")
        UserService.getUserList { (serviceResult) in
            guard case .Error(let error) = serviceResult where error == .NotFound else {
                XCTAssert(false, "Should return with not found error")
                return
            }
            expectation.fulfill()
        }
        waitForExpectationsWithTimeout(5.0, handler: nil)
    }
    
    func testHasUserListResult() {
        // This is an example of a functional test case.
        // Use XCTAssert and related functions to verify your tests produce the correct results.
        
        let userResponseList = [
            ["username": "stefan"],
            ["username": "ben"],
            ["username": "dom"],
            ["username": "w1nfr34k"],
            ["username": "elisa"]
        ]
        
        stub({ (request) -> (Bool) in
            if request.URL!.path!.hasPrefix("/listUsers") {
                return true
            }
            return false
            }, builder: json(userResponseList))
        
        let expectation = expectationWithDescription("Async")
        UserService.getUserList { (serviceResult) in
            guard case .Success(let userList) = serviceResult where userList.count > 0 else {
                XCTAssert(false, "Should return with not found error")
                return
            }
            
            XCTAssertEqual(userList[0].username, "stefan")
            expectation.fulfill()
        }
        waitForExpectationsWithTimeout(5.0, handler: nil)
    }
    
    func testHasUserListResultFromFixtures() {
        // This is an example of a functional test case.
        // Use XCTAssert and related functions to verify your tests produce the correct results.
        
        guard let userResponseData = jsonDataResponse(forFixtureName: "list4Users") else {
            XCTAssert(false, "Fixture not found")
            return
        }
        
        stub({ (request) -> (Bool) in
            if request.URL!.path!.hasPrefix("/listUsers") {
                return true
            }
            return false
            }, builder: jsonData(userResponseData))
        
        let expectation = expectationWithDescription("Async")
        UserService.getUserList { (serviceResult) in
            guard case .Success(let userList) = serviceResult where userList.count > 0 else {
                XCTAssert(false, "Should return with not found error")
                return
            }
            
            XCTAssertEqual(userList[0].username, "stefan")
            expectation.fulfill()
        }
        waitForExpectationsWithTimeout(5.0, handler: nil)
    }

    private func jsonDataResponse(forFixtureName fixtureName: String) -> NSData? {
        guard let path = NSBundle(forClass: TestMeTests.classForCoder()).pathForResource("Fixtures/Users/\(fixtureName)", ofType: "json") else {
            return nil
        }
        
        return NSData(contentsOfFile: path)
    }
}
